@@ -1,6 +1,5 @@
-from os import name
-from random import random
 import numpy as np
+import pandas as pd 
 
 from lost_ds.io.file_man import FileMan
 from lost_ds.geometry.api import LOSTGeometries
@@ -91,22 +90,22 @@ class LOSTDataset(object):
     
     def _parse_data(self):
         # parse lds-serialized data (2D-lists) to numpy array
-        def _parse(x):
+        def _parse(data):
             try: 
-                return np.vstack(x).squeeze()
+                return np.vstack(data).squeeze()
             except:
-                return x
+                return data
         
         parse_keys = [k for k in self.df.keys() if k.endswith('_lds_serialized')]
         if parse_keys:
             col_mapper = dict()
             for k in parse_keys:
-                self.df[k] = self.df[k].apply(lambda x: _parse(x))
+                self.df[k] = self.df[k].apply(_parse)
                 col_mapper[k] = k.replace('_lds_serialized', '')
             self.df.rename(col_mapper, axis=1, inplace=True)
             
         if 'anno_data' in self.df:
-            self.df.anno_data = self.df.anno_data.apply(lambda x: _parse(x))
+            self.df['anno_data'] = self.df['anno_data'].apply(_parse)
         
             
     def to_parquet(self, path, df=None):
@@ -131,7 +130,9 @@ class LOSTDataset(object):
         '''
         if df is None:
             return self.df.copy()
-        else: 
+        elif isinstance(df, LOSTDataset):
+            return df.df
+        elif isinstance(df, pd.DataFrame):
             return df
     
     
@@ -212,7 +213,7 @@ class LOSTDataset(object):
     #
     
     def copy_imgs(self, out_dir, df=None, col='img_path', 
-                  force_overwrite=False):
+                  force_overwrite=False, parallel=-1):
         '''Copy all images of dataset into out_dir
 
         Args:
@@ -222,7 +223,8 @@ class LOSTDataset(object):
         '''
         df = self._get_df(df)
         copy_imgs(df=df, out_dir=out_dir, col=col, 
-                  force_overwrite=force_overwrite, filesystem=self.fileman)
+                  force_overwrite=force_overwrite, filesystem=self.fileman, 
+                  parallel=parallel)
             
     
     def pack_ds(self, out_dir, df=None, cols=['img_path', 'mask_path'], 
@@ -397,7 +399,8 @@ class LOSTDataset(object):
     #   Transformation
     #
     
-    def to_abs(self, df=None, path_col='img_path', verbose=True, inplace=False):
+    def to_abs(self, df=None, path_col='img_path', verbose=True, inplace=False,
+               parallel=-1):
         ''' Transform dataframe to absolute data
         
         Args:
@@ -406,11 +409,12 @@ class LOSTDataset(object):
             inplace (bool): overwrite self.df
         '''
         df = self._get_df(df)
-        df = to_abs(df, path_col, self.fileman, verbose)
+        df = to_abs(df, path_col, self.fileman, verbose, parallel=parallel)
         return self._update_inplace(df, inplace)
     
     
-    def to_rel(self, df=None, path_col='img_path', verbose=True, inplace=False):
+    def to_rel(self, df=None, path_col='img_path', verbose=True, inplace=False,
+               parallel=-1):
         ''' Transform dataframe to relative data
         
         Args:
@@ -419,7 +423,7 @@ class LOSTDataset(object):
             inplace (bool): overwrite self.df
         '''
         df = self._get_df(df)
-        df = to_rel(df, path_col, self.fileman, verbose)
+        df = to_rel(df, path_col, self.fileman, verbose, parallel=parallel)
         return self._update_inplace(df, inplace)
     
     
@@ -673,7 +677,7 @@ class LOSTDataset(object):
     
     
     def vis_semantic_segmentation(self, out_dir, n_classes, palette='dark', 
-                                  seg_path_col='seg_path', df=None):
+                                  seg_path_col='seg_path', df=None, parallel=-1):
         """Visualize the stored semantic segmentations by coloring it
     
         Args:
@@ -687,7 +691,7 @@ class LOSTDataset(object):
         """
         df = self._get_df(df)
         vis_semantic_segmentation(df, out_dir, n_classes, palette, seg_path_col, 
-                                  self.fileman)
+                                  self.fileman, parallel=parallel)
 
 
     #
@@ -695,7 +699,8 @@ class LOSTDataset(object):
     #
     
     def crop_dataset(self, dst_dir, crop_shape=(500, 500), overlap=(0, 0), 
-                     df=None, write_empty=False, fill_value=0, inplace=False):
+                     df=None, write_empty=False, fill_value=0, inplace=False, 
+                     parallel=-1):
         """Crop the entire dataset with fixed crop-shape
 
         Args:
@@ -713,7 +718,8 @@ class LOSTDataset(object):
         df = self._get_df(df)
         df = crop_dataset(df=df, dst_dir=dst_dir, crop_shape=crop_shape, 
                           overlap=overlap, write_empty=write_empty, 
-                          fill_value=fill_value, filesystem=self.fileman)
+                          fill_value=fill_value, filesystem=self.fileman, 
+                          parallel=parallel)
         return self._update_inplace(df, inplace)
     
     
@@ -735,6 +741,30 @@ class LOSTDataset(object):
         return self.cropper.crop_anno(img_path, df, crop_position, im_w, im_h)
     
     
+    def crop_img(self, img, crop_shape=(512, 512), overlap=(0, 0), fill_value=0, 
+                 df=None):
+        '''Crop a 3D image and get a 4D array with crops
+        
+        Args:
+            img (np.array): (H, W, C) Image to crop
+            crop_shape (tuple): (H, W) shape of crops
+            overlap (tuple): (H, W) overlap between crops
+            fill_value (float): pixel-value to fill the area where crops are 
+                out of the image
+            df (pd.DataFrame): dataframe to apply 
+        
+        Returns:
+            (np.ndarray, np.ndarray, tuple): 
+                1. batch of cropped images as 4D-array with shape 
+                    [N, crop_shape[0], crop_shape[1], channels] 
+                2. positions of shape [N, 4] in format [xmin, ymin, xmax, ymax]
+                3. image padding ((top_pad, bot_pad), (left_pad, right_pad))
+        '''
+        df = self._get_df(df)
+        return self.cropper.crop_img(img, crop_shape=crop_shape, 
+                                     overlap=overlap, fill_value=fill_value)
+    
+    
     def crop_components(self, dst_dir, base_labels=-1, lbl_col='anno_lbl', 
                         context=0, df=None, context_alignment=None, 
                         min_size=None, anno_dtype=['polygon'], 
@@ -780,7 +810,7 @@ class LOSTDataset(object):
                               anno_dtypes=['polygon'], use_empty=False, 
                               lbl_col='anno_lbl', dst_path_col='seg_path', 
                               dst_lbl_col='seg_lbl', line_thickness=None, 
-                              radius=None, inplace=False):
+                              radius=None, inplace=False, parallel=-1):
         '''Create semantic segmentations from polygon-annos
     
         Args:
@@ -816,13 +846,13 @@ class LOSTDataset(object):
         df = semantic_segmentation(order, dst_dir, fill_value, df, anno_dtypes,
                                    use_empty, lbl_col, dst_path_col, 
                                    dst_lbl_col, line_thickness, radius, 
-                                   self.fileman)
+                                   self.fileman, parallel=parallel)
         return self._update_inplace(df, inplace)
     
     
     def segmentation_to_lost(self, pixel_mapping, background=0, 
                              seg_key='seg_path', df=None, cast_others=False, 
-                             inplace=False):
+                             inplace=False, parallel=-1):
         '''Create LOST-Annotations from semantic segmentations / pixelmaps 
         
         Args:
@@ -845,7 +875,8 @@ class LOSTDataset(object):
         df = segmentation_to_lost(df, pixel_mapping=pixel_mapping, 
                                       background=background, seg_key=seg_key, 
                                       cast_others=cast_others,
-                                      filesystem=self.fileman)
+                                      filesystem=self.fileman, 
+                                      parallel=parallel)
         return self._update_inplace(df, inplace)
     
     
